#include "dungeon.h"
    /* x=   0 1 2 3 4
     * y=   _   _   _
     * 0 /a\_/a\_/a\
     * \_/a\_/a\_/
     * 1 / \_/ \_/ \
     * \_/.\_/ \_/
     * 2 / \_/.\_/ \
     * \_/ \_/ \_/
     * 3 / \_/ \_/ \
     * \_/ \_/.\_/
     * 4 / \_/ \_/.\
     * \_/ \_/ \_/
     * \_/ \_/
     * 0
     * ____
     * 1 /\  /\ 5
     * /__\/__\
     * \  /\  /
     * 2 \/__\/ 4
     * 3
     */
static int hdx [6] = {0, - 1, - 1, 0, 1, 1};
static int hdy [2] [6] = {{- 1, - 1, 0, 1, 0, - 1}, {- 1, 0, 1, 1, 1, 0}};
static bool _cb_ctype(Creature * c, int ctype);
static bool _cb_injured(Creature * c, int ctype);
static void _hit(Creature * self, int hitmin, int hitmax);
static bool _attack(Creature * self);
static bool _heal(Creature * self);
static void _creature_clone_from_to(Creature * self, Creature * copy);
static int _cmp_cb(void * data1, void * data2);
Dungeon* dungeon_new(void) {
    Dungeon * self;
    land_alloc(self);
    self->creatures = land_array_new();
    land_array_add(self->creatures, NULL) /* reserve spot 0 */;
    return self;
}
void dungeon_destroy(Dungeon * self) {
    land_array_destroy_with_free(self->creatures);
    land_free(self);
}
void dungeon_move(int * xp, int * yp, int d) {
    * yp += hdy [(* xp) & 1] [d];
    * xp += hdx [d];
}
int dungeon_distance(int x1, int y1, int x2, int y2) {
    /* /a\_/c\_/e\
     * \_/b\_/d\_/
     * /f\_/h\_/j\
     * \_/g\_/i\_/
     * /k\_/m\_/o\
     * \_/l\_/n\_/
     * \_/ \_/
     * a  0/0 -> 0/0
     * b  1/0 -> 1/0
     * c  2/0 -> 1/-1
     * d  3/0 -> 2/-1
     * e  4/0 -> 2/-2
     * f  0/1 -> 1/1
     * g  1/1 -> 2/1
     * h  2/1 -> 2/0
     * i  3/1 -> 3/0
     * j  4/1 -> 3/-1
     * k  0/2 -> 2/2
     * l  1/2 -> 3/2
     * m  2/2 -> 3/1
     * n  3/2 -> 4/1
     * o  4/2 -> 4/0
     */
    // first step: convert to diamond coordinates
    int ax = y1 + ((x1 + 1) >> 1);
    int ay = y1 - (x1 >> 1);
    int bx = y2 + ((x2 + 1) >> 1);
    int by = y2 - (x2 >> 1);
    // second step: hex distance
    int dx = bx - ax;
    int dy = by - ay;
    if (dx >= 0) {
        if (dy >= 0) {
            if (dx > dy) {
                return dx;
            }
            return dy;
        }
        return dx - dy;
    }
    if (dy <= 0) {
        if (- dx > - dy) {
            return - dx;
        }
        return - dy;
    }
    return - dx + dy;
}
int dungeon_dir(int x1, int y1, int x2, int y2) {
    int dy = y2 - y1;
    int dx = x2 - x1;
    if (dx == 0) {
        if (dy < 0) {
            return 0;
        }
        else {
            return 3;
        }
    }
    else if (dx > 0) {
        if (dy == 0) {
            return x1 & 1 ? 5 : 4;
        }
        else if (dy > 0) {
            return 4;
        }
        else {
            return 5;
        }
    }
    else {
        if (dy == 0) {
            return x1 & 1 ? 1 : 2;
        }
        else if (dy > 0) {
            return 2;
        }
        else {
            return 1;
        }
    }
}
Tile* dungeon_tile(Dungeon * self, int x, int y) {
    return & self->board [y * board_w + x];
}
Tile* dungeon_tile_dir(Dungeon * self, int x, int y, int d) {
    dungeon_move(& x, & y, d);
    return & self->board [y * board_w + x];
}
Creature* dungeon_creature(Dungeon * self, int cid) {
    return land_array_get(self->creatures, cid);
}
int dungeon_kind(Dungeon * self, int x, int y) {
    return self->board [y * board_w + x].kind;
}
void dungeon_clear(Dungeon * self) {
    for (int y = 0; y < board_h; y += 1) {
        for (int x = 0; x < board_w; x += 1) {
            bool border = x <= 1 || y <= 2 || x >= board_w - 2 || y == board_h - 1;
            Tile * tile = dungeon_tile(self, x, y);
            tile->kind = border ? 0 : 2;
            tile->light = 0;
            tile->creature = 0;
            tile->howfar = 0;
            tile->path_step = 0;
        }
    }
}
float dungeon_get_closest_light(Dungeon * self, int xp, int yp) {
    float closest = board_w * board_h;
    for (int y = 0; y < board_h; y += 1) {
        for (int x = 0; x < board_w; x += 1) {
            Tile * tile = dungeon_tile(self, x, y);
            if (tile->kind == 2 && tile->kid == game->wall_torch->kid) {
                int d = dungeon_distance(x, y, xp, yp);
                if (d < closest) {
                    closest = d;
                }
            }
        }
    }
    return closest;
}
void dungeon_light(Dungeon * self) {
    for (int y = 0; y < board_h; y += 1) {
        for (int x = 0; x < board_w; x += 1) {
            float d = dungeon_get_closest_light(self, x, y);
            float l = (1 + 7 - d) / 7;
            if (l < 0) {
                l = 0;
            }
            if (l > 1) {
                l = 1;
            }
            self->board [y * board_w + x].light = l;
        }
    }
}
Creature* dungeon_creature_new(Dungeon * self, Kind * kind, int x, int y) {
    Creature * c;
    land_alloc(c);
    c->dungeon = self;
    c->kid = kind->kid;
    c->x = x;
    c->y = y;
    c->px = x;
    c->py = y;
    c->d = c->pd = 4;
    c->cid = land_array_count(self->creatures);
    land_array_add(self->creatures, c);
    c->hidden = 1;
    c->name = kind_create_name(kind);
    dungeon_creature_update_stats(c);
    if (c->ctype == 3) {
        c->to_exit = 1;
    }
    c->hp = c->hp_max;
    c->mp = c->mp_max;
    return c;
}
void dungeon_creature_update_stats(Creature * c) {
    Kind * kind = kind_get(c->kid);
    int old_hp_max = c->hp_max;
    if (kind == game->knight) {
        c->hp_max = 35;
        c->hitmin = 4;
        c->hitmax = 6;
        c->ctype = 3;
        if (c->equip [Main] == game->sword->kid) {
            c->hitmin = 8;
            c->hitmax = 12;
        }
        if (c->equip [Head] == game->helmet->kid) {
            c->hp_max += 40;
        }
        if (c->equip [Off] == game->shield->kid) {
            c->hp_max += 25;
        }
    }
    if (kind == game->archer) {
        c->hp_max = 35;
        c->hitmin = 2;
        c->hitmax = 6;
        c->ctype = 3;
        if (c->equip [Main] == game->bow->kid) {
            c->hitmax = 12;
        }
        if (c->equip [Neck] == game->quiver->kid) {
            c->hitmin = 6;
        }
        if (c->equip [Head] == game->helmet2->kid) {
            c->hp_max += 40;
        }
    }
    if (kind == game->mage) {
        c->hp_max = 35;
        c->hitmin = 3;
        c->hitmax = 9;
        c->ctype = 3;
        if (c->equip [Off] == game->book->kid) {
            c->hitmin = 9;
        }
        if (c->equip [Main] == game->wand->kid) {
            c->hitmax = 19;
        }
        if (c->equip [Head] == game->hat->kid) {
            c->hp_max += 40;
        }
    }
    if (kind == game->elf) {
        c->hp_max = 75;
        c->hitmin = 2;
        c->hitmax = 8;
        c->ctype = 3;
        c->mp_max = 100;
    }
    if (kind == game->rat) {
        c->hp_max = 50;
        c->hitmin = 5;
        c->hitmax = 10;
        c->ctype = 4;
    }
    if (kind == game->troll) {
        c->hp_max = 250;
        c->hitmin = 2;
        c->hitmax = 20;
        c->ctype = 4;
    }
    if (kind == game->troll2) {
        c->hp_max = 600;
        c->hitmin = 4;
        c->hitmax = 40;
        c->ctype = 4;
    }
    if (kind == game->priest) {
        c->hp_max = 150;
        c->hitmin = 10;
        c->hitmax = 30;
        c->ctype = 4;
        c->pot_tel = 3;
    }
    if (kind == game->dragon) {
        c->hp_max = 1800;
        c->hitmin = 20;
        c->hitmax = 60;
        c->ctype = 4;
    }
    if (c->hp_max > old_hp_max) {
        c->hp += c->hp_max - old_hp_max;
    }
    if (c->hp > c->hp_max) {
        c->hp = c->hp_max;
    }
}
int count_kinds(Dungeon * self, int x, int y, int k) {
    int count = 0;
    for (int d = 0; d < 6; d += 1) {
        int x2 = x, y2 = y;
        dungeon_move(& x2, & y2, d);
        if (dungeon_kind(self, x2, y2) == k) {
            count += 1;
        }
    }
    return count;
}
bool is_walkable(Dungeon * self, int x, int y, int d) {
    int x2 = x, y2 = y;
    dungeon_move(& x2, & y2, d);
    Tile * tile = dungeon_tile(self, x2, y2);
    if (tile->kind == 1 && tile->creature == 0) {
        return 1;
    }
    return 0;
}
int count_walkables(Dungeon * self, int x, int y) {
    int count = 0;
    for (int d = 0; d < 6; d += 1) {
        if (is_walkable(self, x, y, d)) {
            count += 1;
        }
    }
    return count;
}
void check_attack(Creature * self) {
    int checkd [6] = {0, 1, 5, 2, 4, 3};
    for (int i = 0; i < 6; i += 1) {
        int x2 = self->x;
        int y2 = self->y;
        int d = (self->d + checkd [i]) % 6;
        dungeon_move(& x2, & y2, d);
        Tile * tile = dungeon_tile(self->dungeon, x2, y2);
        if (tile->creature) {
            Creature * c = dungeon_creature(self->dungeon, tile->creature);
            if (c->ctype != self->ctype) {
                self->d = d;
                Kind * k = kind_get(self->kid);
                if (k->sound_attack) {
                    data_playq(k->sound_attack);
                }
                self->fighting = c->cid;
                if (! c->fighting) {
                    Kind * kc = kind_get(c->kid);
                    if (kc->sound_attack) {
                        data_playq(kc->sound_attack);
                    }
                    c->fighting = self->cid;
                }
                return ;
            }
        }
    }
}
// checks adjacent, then 2, 3, ... radius
bool check_ranged(Creature * self, int x, int y, int d, int n, int radius, bool(* callback)(Creature * c, int p), int p, int * fx, int * fy) {
    int checkd [6] = {0, 1, 5, 2, 4, 3};
    for (int i = 0; i < n; i += 1) {
        int x2 = x;
        int y2 = y;
        int d2 = (d + checkd [i]) % 6;
        dungeon_move(& x2, & y2, d2);
        Tile * tile = dungeon_tile(self->dungeon, x2, y2);
        if (tile->kind == 1) {
            if (tile->creature) {
                Creature * c = dungeon_creature(self->dungeon, tile->creature);
                if (callback(c, p)) {
                    * fx = x2;
                    * fy = y2;
                    return 1;
                }
            }
            if (radius > 1) {
                if (check_ranged(self, x2, y2, d2, 3, radius - 1, callback, p, fx, fy)) {
                    return 1;
                }
            }
        }
    }
    return 0;
}
static bool _cb_ctype(Creature * c, int ctype) {
    return c->ctype == ctype && ! c->dead;
}
static bool _cb_injured(Creature * c, int ctype) {
    return c->ctype == ctype && c->hp < c->hp_max && ! c->dead;
}
void check_attack_ranged(Creature * self, int radius, int ctype) {
    int fx, fy;
    if (check_ranged(self, self->x, self->y, self->d, 6, radius, _cb_ctype, ctype, & fx, & fy)) {
        self->d = dungeon_dir(self->x, self->y, fx, fy);
        Tile * tile = dungeon_tile(self->dungeon, fx, fy);
        self->fighting = tile->creature;
        Creature * c = dungeon_creature(self->dungeon, self->fighting);
        if (! c->fighting) {
            c->fighting = self->cid;
        }
    }
}
void check_healing_ranged(Creature * self, int radius, int ctype) {
    int fx, fy;
    if (check_ranged(self, self->x, self->y, self->d, 6, radius, _cb_injured, ctype, & fx, & fy)) {
        self->d = dungeon_dir(self->x, self->y, fx, fy);
        Tile * tile = dungeon_tile(self->dungeon, fx, fy);
        self->healing = tile->creature;
    }
}
static void _hit(Creature * self, int hitmin, int hitmax) {
    int hit = land_rand(hitmin, hitmax);
    self->hp -= hit;
    print("%d gets hit %d -> %d/%d", self->cid, hit, self->hp, self->hp_max);
    if (self->hp <= 0) {
        dungeon_die(self);
    }
}
bool dungeon_teleport(Creature * self, int r) {
    int x = self->x;
    int y = self->y;
    int d = land_rand(0, 5);
    for (int i = 0; i < r; i += 1) {
        dungeon_move(& x, & y, d);
    }
    d = (d + 2) % 6;
    for (int j = 0; j < 6; j += 1) {
        for (int i = 0; i < r; i += 1) {
            dungeon_move(& x, & y, d);
            Tile * tile = dungeon_tile(self->dungeon, x, y);
            if (tile->creature == 0 && tile->kind == 1) {
                dungeon_remove(self);
                self->x = x;
                self->y = y;
                dungeon_appear(self);
                Kind * k = kind_get(self->kid);
                if (k->sound_special) {
                    data_playq(k->sound_special);
                }
                return 1;
            }
        }
        d = (d + 1) % 6;
    }
    return 0;
}
void dungeon_remove(Creature * self) {
    Tile * tile = dungeon_tile(self->dungeon, self->x, self->y);
    tile->creature = 0;
}
void dungeon_appear(Creature * self) {
    Tile * tile = dungeon_tile(self->dungeon, self->x, self->y);
    tile->creature = self->cid;
}
static bool _attack(Creature * self) {
    Creature * c = dungeon_creature(self->dungeon, self->fighting);
    if (c->dead) {
        self->fighting = 0;
        return 0;
    }
    self->d = dungeon_dir(self->x, self->y, c->x, c->y);
    bool in_range = 0;
    int distance = dungeon_distance(self->x, self->y, c->x, c->y);
    if (distance == 1) {
        in_range = 1;
    }
    else if (distance == 2) {
        if (self->kid == game->archer->kid || self->kid == game->priest->kid || self->kid == game->mage->kid) {
            in_range = 1;
        }
        else {
            // even if we can reach them with a step we don't want the
            // fighting animation yet
            self->fighting = 0;
            // see if moving in a direction gets us to distance 1
            for (int d = 0; d < 6; d += 1) {
                int x2 = self->x;
                int y2 = self->y;
                dungeon_move(& x2, & y2, d);
                Tile * tile = dungeon_tile(self->dungeon, x2, y2);
                if (tile->creature == 0 && tile->kind == 1) {
                    if (dungeon_distance(x2, y2, c->x, c->y) == 1) {
                        creature_move(self, d);
                        return 1;
                    }
                }
            }
            return 0;
        }
    }
    else if (distance == 3) {
        if (self->kid == game->mage->kid) {
            in_range = 1;
        }
        else {
            self->fighting = 0;
            return 0;
        }
    }
    else {
        self->fighting = 0;
        return 0;
    }
    if (in_range) {
        if (distance > 1) {
            if (self->kid == game->mage->kid) {
                game_projectile(game->fireball, self->cid, c->cid);
            }
            if (self->kid == game->archer->kid) {
                game_projectile(game->arrow, self->cid, c->cid);
            }
            if (self->kid == game->priest->kid) {
                game_projectile(game->missile, self->cid, c->cid);
            }
        }
        Kind * k = kind_get(self->kid);
        if (k->sound_fight) {
            data_playq_vol(k->sound_fight, 0.5);
        }
        _hit(c, self->hitmin, self->hitmax);
        if (c->hp <= 0) {
            self->fighting = 0;
            if (self->kid == game->priest->kid) {
                self->hp += 30;
                if (self->hp > self->hp_max) {
                    self->hp = self->hp_max;
                }
            }
        }
        return 1;
    }
    return 0;
}
void dungeon_die(Creature * self) {
    self->hp = 0;
    self->dead = 1;
    self->pd = self->d;
    self->fighting = 0;
    int r = land_rand(0, 3);
    self->equip [r] = 0;
    Kind * k = kind_get(self->kid);
    if (k->sound_down) {
        data_playq(k->sound_down);
    }
    dungeon_remove(self);
}
static bool _heal(Creature * self) {
    if (self->mp == 0) {
        self->healing = 0;
        return 0;
    }
    Creature * c = dungeon_creature(self->dungeon, self->healing);
    self->d = dungeon_dir(self->x, self->y, c->x, c->y);
    int distance = dungeon_distance(self->x, self->y, c->x, c->y);
    if (distance <= 3 && ! c->dead) {
        if (c->hp < c->hp_max) {
            Kind * k = kind_get(self->kid);
            if (k->sound_special) {
                data_playq(k->sound_special);
            }
            int h = c->hp_max - c->hp;
            int roll = land_rand(10, 25);
            if (roll > h) {
                roll = h;
            }
            if (roll > self->mp) {
                roll = self->mp;
            }
            self->mp -= roll;
            c->hp += roll;
            return 1;
        }
    }
    self->healing = 0;
    return 0;
}
void creature_move(Creature * self, int d) {
    dungeon_remove(self);
    dungeon_move(& self->x, & self->y, d);
    self->d = d;
    dungeon_appear(self);
}
bool check_equipment(Creature * self) {
    if (self->ctype == 4) {
        // troll self heal
        if (self->kid == game->troll->kid || self->kid == game->troll2->kid) {
            if (self->hp < self->hp_max) {
                self->hp++;
                return 0 /* note: rest of turn continues */;
            }
        }
        // demon reanimate
        if (self->kid == game->priest->kid) {
            {
                LandArrayIterator __iter0__ = LandArrayIterator_first(self->dungeon->creatures);
                for (Creature * c = LandArrayIterator_item(self->dungeon->creatures, &__iter0__); LandArrayIterator_next(self->dungeon->creatures, &__iter0__); c = LandArrayIterator_item(self->dungeon->creatures, &__iter0__)) {
                    if (c && c->dead && c->ctype == 4 && ! c->hidden) {
                        if (dungeon_distance(c->x, c->y, self->x, self->y) <= 1) {
                            c->hidden = 1;
                            c->dead = 0;
                            c->hp = 30 /* reanimated have almost no hp */;
                            self->hp += 30;
                            // but demon receives hp
                            if (self->hp > self->hp_max) {
                                self->hp = self->hp_max;
                            }
                            return 1;
                        }
                    }
                }
            }
        }
        // rat revive rat
        if (self->kid == game->rat->kid) {
            {
                LandArrayIterator __iter0__ = LandArrayIterator_first(self->dungeon->creatures);
                for (Creature * c = LandArrayIterator_item(self->dungeon->creatures, &__iter0__); LandArrayIterator_next(self->dungeon->creatures, &__iter0__); c = LandArrayIterator_item(self->dungeon->creatures, &__iter0__)) {
                    if (c && c->dead && c->kid == game->rat->kid && ! c->hidden) {
                        if (dungeon_distance(c->x, c->y, self->x, self->y) == 0) {
                            c->hidden = 1;
                            c->dead = 0;
                            c->hp = 30 /* reanimated have almost no hp */;
                            return 1;
                        }
                    }
                }
            }
        }
        return 0;
    }
    if ((self->hp < self->hp_max - 30 || self->hp < self->hp_max / 2) && self->pot_hp) {
        self->pot_hp -= 1;
        self->hp += 40;
        if (self->hp > self->hp_max) {
            self->hp = self->hp_max;
        }
        game_use(game->potion_hp, self->x, self->y);
        data_playq(DataPotion);
        return 1;
    }
    if (self->mp < self->mp_max - 30 && self->pot_mp) {
        self->pot_mp -= 1;
        self->mp += 40;
        if (self->mp > self->mp_max) {
            self->mp = self->mp_max;
        }
        game_use(game->potion_mp, self->x, self->y);
        data_playq(DataPotion);
        return 1;
    }
    if (self->pot_res) {
        {
            LandArrayIterator __iter0__ = LandArrayIterator_first(self->dungeon->creatures);
            for (Creature * c = LandArrayIterator_item(self->dungeon->creatures, &__iter0__); LandArrayIterator_next(self->dungeon->creatures, &__iter0__); c = LandArrayIterator_item(self->dungeon->creatures, &__iter0__)) {
                if (! c || c->ctype != 3) {
                    continue;
                }
                if (dungeon_distance(self->x, self->y, c->x, c->y) <= 1) {
                    // resurrect
                    if (c->dead && ! c->hidden) {
                        self->pot_res -= 1;
                        c->hidden = 1;
                        c->dead = 0;
                        c->hp = c->hp_max;
                        game_use(game->potion_res, self->x, self->y);
                        data_playq(DataPotion);
                        return 1;
                    }
                }
            }
        }
    }
    return 0;
}
bool buy_equipment(Creature * self, Kind * who, int where, Kind * what) {
    if (self->kid == who->kid) {
        if (self->equip [where] == 0) {
            self->equip [where] = what->kid;
            dungeon_creature_update_stats(self);
            game_buy(what, self->x, self->y);
            return 1;
        }
    }
    return 0;
}
bool check_vendor(Creature * self) {
    if (self->ctype != 3) {
        return 0;
    }
    for (int d = 0; d < 6; d += 1) {
        int x = self->x, y = self->y;
        dungeon_move(& x, & y, d);
        Tile * tile = dungeon_tile(self->dungeon, x, y);
        if (tile->kind == 2 && tile->kid == game->vendor_shield->kid) {
            // shield vendor
            if (buy_equipment(self, game->knight, Off, game->shield)) {
                return 1;
            }
            if (buy_equipment(self, game->archer, Neck, game->quiver)) {
                return 1;
            }
            if (buy_equipment(self, game->mage, Off, game->book)) {
                return 1;
            }
        }
        if (tile->kind == 2 && tile->kid == game->vendor_head->kid) {
            // helmet vendor
            if (buy_equipment(self, game->knight, Head, game->helmet)) {
                return 1;
            }
            if (buy_equipment(self, game->archer, Head, game->helmet2)) {
                return 1;
            }
            if (buy_equipment(self, game->mage, Head, game->hat)) {
                return 1;
            }
        }
        if (tile->kind == 2 && tile->kid == game->vendor_potion->kid) {
            // potions vendor
            if (self->pot_hp == 0) {
                self->pot_hp += 1;
                game_buy(game->potion_hp, x, y);
                return 1;
            }
            if (self->kid == game->elf->kid && self->pot_mp == 0) {
                self->pot_mp += 1;
                game_buy(game->potion_mp, x, y);
                return 1;
            }
            if (self->kid == game->elf->kid && self->pot_res == 0) {
                self->pot_res += 1;
                game_buy(game->potion_res, x, y);
                return 1;
            }
        }
        if (tile->kind == 2 && tile->kid == game->vendor_smith->kid) {
            // weapons vendor
            if (buy_equipment(self, game->knight, Main, game->sword)) {
                return 1;
            }
            if (buy_equipment(self, game->archer, Main, game->bow)) {
                return 1;
            }
            if (buy_equipment(self, game->mage, Main, game->wand)) {
                return 1;
            }
        }
    }
    return 0;
}
bool check_trap(Creature * self) {
    Tile * tile = dungeon_tile(self->dungeon, self->x, self->y);
    if (tile->kid == game->trap_box->kid && self->ctype == 3) {
        tile->kid = game->ground [0]->kid;
        self->dungeon->modified = 1;
        bool r = 1;
        if (self->to_loot) {
            self->to_loot = 0;
            r = 0;
        }
        else {
            ;
        }
        game_buy(game->chest, self->x, self->y);
        return r;
    }
    if (tile->kid == game->trap_poison->kid) {
        tile->kid = game->ground [0]->kid;
        self->dungeon->modified = 1;
        if (self->kid != game->troll->kid && self->kid != game->troll2->kid) {
            _hit(self, 20, 30);
            return 1;
        }
        return 0;
    }
    if (tile->kid == game->trap_fire->kid) {
        tile->kid = game->ground [0]->kid;
        self->dungeon->modified = 1;
        if (self->kid != game->dragon->kid) {
            _hit(self, 20, 30);
            return 1;
        }
        return 0;
    }
    if (tile->kid == game->trap_lightning->kid) {
        if (self->ctype == 4) {
            tile->kid = game->ground [0]->kid;
            self->dungeon->modified = 1;
            _hit(self, 20, 30);
            return 1;
        }
        return 0;
    }
    return 0;
}
static void _creature_clone_from_to(Creature * self, Creature * copy) {
    copy->hp = self->hp;
    copy->hp_max = self->hp_max;
    copy->mp = self->mp;
    copy->mp_max = self->mp_max;
    copy->hitmin = self->hitmin;
    copy->hitmax = self->hitmax;
    for (int i = 0; i < 4; i += 1) {
        copy->equip [i] = self->equip [i];
    }
    copy->pot_hp = self->pot_hp;
    copy->pot_mp = self->pot_mp;
    copy->pot_res = self->pot_res;
    if (copy->name) {
        land_free(copy->name);
    }
    copy->name = land_strdup(self->name);
}
void hero_step(Creature * self) {
    self->px = self->x;
    self->py = self->y;
    self->pd = self->d;
    int back = (self->d + 3) % 6;
    bool turnaround = check_trap(self);
    if (self->dead) {
        return ;
    }
    if (self->kid == game->priest->kid) {
        if (self->hp < 30) {
            if (self->pot_tel > 0) {
                self->fighting = 0;
                // try distance 4 first else 3
                if (dungeon_teleport(self, 4)) {
                    self->hp += 60;
                    self->pot_tel -= 1;
                    return ;
                }
                if (dungeon_teleport(self, 3)) {
                    self->hp += 60;
                    self->pot_tel -= 1;
                    return ;
                }
            }
        }
    }
    if (self->fighting) {
        if (_attack(self)) {
            return ;
        }
    }
    if (self->healing) {
        if (_heal(self)) {
            return ;
        }
    }
    check_attack(self);
    if (self->fighting) {
        return ;
    }
    if (self->kid == game->archer->kid || self->kid == game->priest->kid) {
        int e = 4;
        if (self->ctype == 4) {
            e = 3;
        }
        check_attack_ranged(self, 2, e);
        if (self->fighting) {
            return ;
        }
    }
    if (self->kid == game->mage->kid) {
        int e = 4;
        if (self->ctype == 4) {
            e = 3;
        }
        check_attack_ranged(self, 3, e);
        if (self->fighting) {
            return ;
        }
    }
    if (self->kid == game->elf->kid && self->mp > 0) {
        check_healing_ranged(self, 3, 3);
        if (self->healing) {
            return ;
        }
    }
    if (check_equipment(self)) {
        return ;
    }
    if (check_vendor(self)) {
        return ;
    }
    Tile * ptile = dungeon_tile(self->dungeon, self->px, self->py);
    int i = 0;
    int w = count_walkables(self->dungeon, self->x, self->y);
    bool wb = is_walkable(self->dungeon, self->x, self->y, back);
    if (self->to_exit) {
        if (self->x == self->dungeon->ex && self->y == self->dungeon->ey) {
            self->to_exit = 0;
            if (self->ctype == 3) {
                game_unlock_level();
                // advance to next level
                if (self->dungeon->level < 9) {
                    self->hidden = 1;
                    self->dead = 1;
                    dungeon_remove(self);
                    // in reality we create a duplicate
                    Dungeon * down = game->levels [self->dungeon->level + 1];
                    Creature * c = dungeon_creature_new(down, kind_get(self->kid), down->sx, down->sy);
                    _creature_clone_from_to(self, c);
                    data_play(DataWin);
                }
                return ;
            }
        }
    }
    else {
        if (self->x == self->dungeon->sx && self->y == self->dungeon->sy) {
            self->to_exit = 1;
        }
    }
    if (turnaround && wb) {
        creature_move(self, back);
        return ;
    }
    if (w > 2 || (! wb && w > 1)) {
        // move to loot box
        int lootd [] = {0, 1, 5, 2, 3};
        for (int di = 0; di < 5; di += 1) {
            int d = (self->d + lootd [di]) % 6;
            Tile * tile = dungeon_tile_dir(self->dungeon, self->x, self->y, d);
            if (tile->creature == 0 && tile->kid == game->trap_box->kid) {
                creature_move(self, d);
                self->to_loot = 1;
                return ;
            }
        }
    }
    if (w == 0) {
        // trapped
        self->d = land_rand(0, 5);
        return ;
    }
    else if (w == 1) {
        i = 0;
    }
    else {
        // Note: the back direction may be blocked since the last step,
        // e.g. someone walking behind us
        if (wb) {
            i = land_rand(0, w - 2) /* exclude back if we can walk back */;
        }
        else {
            i = land_rand(0, w - 1);
        }
    }
    if (w > 2) {
        if (ptile->path_step && ! turnaround) {
            int next_step = ptile->path_step;
            if (self->to_exit) {
                next_step += 1;
            }
            else {
                next_step -= 1;
            }
            // if we are on the main path, 50% of the time stay on it
            if (land_rand(0, 1) == 0) {
                // follow the path
                for (int d = 0; d < 6; d += 1) {
                    Tile * tile = dungeon_tile_dir(self->dungeon, self->x, self->y, d);
                    if (tile->kind == 1 && tile->creature == 0 && tile->path_step == next_step) {
                        creature_move(self, d);
                        return ;
                    }
                }
            }
        }
        // if we're in a party 50% of the time try to get close to the leader
        if (self->leader && land_rand(0, 1) == 0) {
            Creature * l = dungeon_creature(self->dungeon, self->leader);
            int mindist = 1000;
            int mind = 6;
            for (int d = 0; d < 6; d += 1) {
                int x2 = self->x, y2 = self->y;
                dungeon_move(& x2, & y2, d);
                Tile * tile = dungeon_tile(self->dungeon, x2, y2);
                if (tile->creature == 0 && tile->kind == 1) {
                    int dist = dungeon_distance(x2, y2, l->x, l->y);
                    if (dist < mindist) {
                        mindist = dist;
                        mind = d;
                    }
                }
            }
            if (mind != 6) {
                creature_move(self, mind);
                return ;
            }
        }
    }
    for (int d = 0; d < 6; d += 1) {
        if (w > 1 && wb && d == back) {
            continue;
        }
        if (is_walkable(self->dungeon, self->x, self->y, d)) {
            if (i == 0) {
                creature_move(self, d);
                return ;
            }
            i -= 1;
        }
    }
}
void dungeon_use(int kid) {
    for (int uid = 0; uid < 3; uid += 1) {
        Kind * k = kind_get(kid);
        game->used [uid] += k->upkeep [uid];
        game->used_stats [uid] [kid] += k->upkeep [uid];
    }
}
void dungeon_step(Dungeon * self) {
    self->ticks++;
    bool reveal_hero = 0;
    bool reveal_monster = 0;
    reveal_hero = 1;
    reveal_monster = 1;
    {
        LandArrayIterator __iter0__ = LandArrayIterator_first(self->creatures);
        for (Creature * c = LandArrayIterator_item(self->creatures, &__iter0__); LandArrayIterator_next(self->creatures, &__iter0__); c = LandArrayIterator_item(self->creatures, &__iter0__)) {
            if (! c) {
                continue;
            }
            if (c->dead) {
                continue;
            }
            if (c->hidden) {
                if ((c->ctype == 3 && reveal_hero) || (c->ctype == 4 && reveal_monster)) {
                    Tile * tile = dungeon_tile(self, c->x, c->y);
                    if (tile->creature == 0) {
                        tile->creature = c->cid;
                        if (c->ctype == 3) {
                            reveal_hero = 0;
                        }
                        if (c->ctype == 4) {
                            reveal_monster = 0;
                        }
                        c->hidden = 0;
                        print("New! %d %d %d", c->x, c->y, c->cid);
                    }
                }
                continue;
            }
            dungeon_use(c->kid);
            if (c->ctype == 3) {
                hero_step(c);
            }
            if (c->ctype == 4) {
                hero_step(c);
            }
        }
    }
    for (int y = 0; y < board_h; y += 1) {
        for (int x = 0; x < board_w; x += 1) {
            Tile * tile = dungeon_tile(self, x, y);
            if (tile->kid) {
                dungeon_use(tile->kid);
            }
        }
    }
}
static int _cmp_cb(void * data1, void * data2) {
    PathNode * n1 = data1;
    PathNode * n2 = data2;
    if (n1->steps + n1->estimate < n2->steps + n2->estimate) {
        return - 1;
    }
    if (n1->steps + n1->estimate > n2->steps + n2->estimate) {
        return 1;
    }
    return 0;
}
int dungeon_path(Dungeon * self, int x1, int y1, int x2, int y2, int maxlen, int * dirs) {
    int n = board_w * board_h;
    PathNode nodes [n];
    char mark [n];
    memset(mark, 0, n);
    int used = 0;
    if (! self->q) {
        self->q = land_queue_new(_cmp_cb);
    }
    land_queue_clear(self->q);
    PathNode * node0 = nodes + used++;
    node0->steps = 0;
    node0->x = x1;
    node0->y = y1;
    node0->d = 0;
    node0->estimate = dungeon_distance(x1, y1, x2, y2);
    node0->parent = NULL;
    land_queue_add(self->q, node0);
    while (! land_queue_is_empty(self->q)) {
        PathNode * node = land_queue_pop(self->q);
        for (int d = 0; d < 6; d += 1) {
            int x = node->x;
            int y = node->y;
            dungeon_move(& x, & y, d);
            if (mark [y * board_w + x] == 1) {
                continue;
            }
            if (x == x2 && y == y2) {
                // Found it!
                PathNode * trace = node;
                int i = trace->steps;
                dirs [i] = d;
                while (i > 0) {
                    i--;
                    dirs [i] = trace->d;
                    trace = trace->parent;
                }
                return node->steps + 1;
            }
            mark [y * board_w + x] = 1;
            Tile * tile = dungeon_tile(self, x, y);
            if (tile->kind != 1) {
                continue;
            }
            PathNode * node1 = nodes + used++;
            node1->steps = node->steps + 1;
            node1->x = x;
            node1->y = y;
            node1->d = d;
            node1->parent = node;
            node1->estimate = dungeon_distance(x1, y1, x2, y2);
            if (node1->steps + node1->estimate < maxlen) {
                land_queue_add(self->q, node1);
            }
            else {
                print("path exceeded");
            }
        }
    }
    return 0;
}
void dungeon_add_quest(Dungeon * dungeon, int q, bool spawn) {
    Creature * c1 = game_spawn_hero(dungeon, game->knight, spawn);
    if (q >= 2) {
        Creature * c2 = game_spawn_hero(dungeon, game->archer, spawn);
        c2->leader = c1->cid;
    }
    if (q >= 3) {
        Creature * c3 = game_spawn_hero(dungeon, game->elf, spawn);
        c3->leader = c1->cid;
    }
    if (q >= 4) {
        Creature * c4 = game_spawn_hero(dungeon, game->mage, spawn);
        c4->leader = c1->cid;
    }
}
