#ifndef _DUNGEON_
#define _DUNGEON_
typedef struct Tile Tile;
typedef enum EquipmentSlot EquipmentSlot;
typedef enum UtilityType UtilityType;
typedef struct Creature Creature;
typedef struct Dungeon Dungeon;
typedef struct PathNode PathNode;
#include "common.h"
#include "data.h"
#include "render.h"
#include "create.h"
#define board_w 39
#define board_h 35
struct Tile {
    int kind /* 0 = border, 1 = ground, 2 = wall */;
    int kid;
    float light /* 0 = dark, 1 = bright */;
    int creature;
    int howfar;
    int path_step;
};
enum EquipmentSlot {
    Main,
    Off,
    Head,
    Neck
};
enum UtilityType {
    Water,
    Gas,
    Electric
};
struct Creature {
    int cid;
    Dungeon * dungeon;
    int ctype /* 1 = critter, 2 = npc, 3 = hero, 4 = monster */;
    int kid;
    int x, y;
    int d;
    int px, py, pd;
    bool hidden : 1;
    bool dead : 1;
    int fighting /* cid */;
    int healing /* cid */;
    int hp, hp_max;
    int mp, mp_max;
    int hitmin, hitmax;
    int leader /* 0 if solo, else the party leader */;
    bool to_exit;
    bool to_loot;
    int equip [4] /* kid: main, off, head, neck */;
    int pot_hp;
    int pot_mp;
    int pot_res;
    int pot_tel;
    char * name;
};
struct Dungeon {
    Tile board [board_h * board_w];
    LandArray * creatures;
    int sx, sy;
    int ex, ey;
    int ticks;
    LandQueue * q;
    int floors;
    int level /* 0 = overworld, 1 = level 1, ... */;
    bool modified;
};
struct PathNode {
    int steps;
    int estimate;
    int d;
    int x, y;
    PathNode * parent;
};
Dungeon* dungeon_new(void);
void dungeon_destroy(Dungeon * self);
void dungeon_move(int * xp, int * yp, int d);
int dungeon_distance(int x1, int y1, int x2, int y2);
int dungeon_dir(int x1, int y1, int x2, int y2);
Tile* dungeon_tile(Dungeon * self, int x, int y);
Tile* dungeon_tile_dir(Dungeon * self, int x, int y, int d);
Creature* dungeon_creature(Dungeon * self, int cid);
int dungeon_kind(Dungeon * self, int x, int y);
void dungeon_clear(Dungeon * self);
float dungeon_get_closest_light(Dungeon * self, int xp, int yp);
void dungeon_light(Dungeon * self);
Creature* dungeon_creature_new(Dungeon * self, Kind * kind, int x, int y);
void dungeon_creature_update_stats(Creature * c);
int count_kinds(Dungeon * self, int x, int y, int k);
bool is_walkable(Dungeon * self, int x, int y, int d);
int count_walkables(Dungeon * self, int x, int y);
void check_attack(Creature * self);
bool check_ranged(Creature * self, int x, int y, int d, int n, int radius, bool(* callback)(Creature * c, int p), int p, int * fx, int * fy);
void check_attack_ranged(Creature * self, int radius, int ctype);
void check_healing_ranged(Creature * self, int radius, int ctype);
bool dungeon_teleport(Creature * self, int r);
void dungeon_remove(Creature * self);
void dungeon_appear(Creature * self);
void dungeon_die(Creature * self);
void creature_move(Creature * self, int d);
bool check_equipment(Creature * self);
bool buy_equipment(Creature * self, Kind * who, int where, Kind * what);
bool check_vendor(Creature * self);
bool check_trap(Creature * self);
void hero_step(Creature * self);
void dungeon_use(int kid);
void dungeon_step(Dungeon * self);
int dungeon_path(Dungeon * self, int x1, int y1, int x2, int y2, int maxlen, int * dirs);
void dungeon_add_quest(Dungeon * dungeon, int q, bool spawn);
#endif
