#ifndef _RENDER_
#define _RENDER_
typedef struct Render Render;
typedef struct Object Object;
typedef struct Kind Kind;
#include "main.h"
#include "dungeon.h"
#include "data.h"
struct Render {
    LandCamera * cam;
    LandVector light;
    Land4x4Matrix projection;
    char * vertex_shader;
    char * fragment_shader;
    int stats_triangles;
    float minz;
    bool more /* set to False to initialize shaders */;
};
struct Object {
    Creature * creature;
    Land4x4Matrix matrix;
    Kind * kind;
    float height;
    float light;
    float xpix, ypix;
    bool dead;
    bool frozen;
    Creature * home;
    uint64_t expire_t;
    Kind * equip [4] /* main, off, head, neck */;
    Shadows * shadows_normal;
    Shadows * shadows_attack;
};
extern Render render;
struct Kind {
    int kid;
    int cost;
    bool is_creature;
    Data3DModel * normal;
    Data3DModel * attack;
    Data3DModel * dead;
    int upkeep [3];
    char * name;
    int sound_attack;
    int sound_fight;
    int sound_special;
    int sound_down;
};
Kind* kind_get(int kid);
Kind* render_kind_new(str anim, str anim_attack, str anim_dead, float scale, bool moving);
void kind_give_name(Kind * k, str name);
Kind* render_kind_new_static(str name, float scale);
Kind* render_kind_new_anim(str name, str name_attack, str name_dead, float scale);
Kind* render_kind_new_shadow(str name, float scale);
void render_clear(void);
void render_3d_projection(void);
void render_2d_projection(void);
void hexa(float x, float y, float w, float h);
void hexf(float x, float y, float w, float h);
void get_hexpos(int x, int y, float * sx, float * sz);
Object* add_static_object(float x, float y, float z, Kind * k, float angle);
Object* render_add_moving_object(Creature * creature);
void object_destroy(Object * o);
Object* render_add_particle_to(Kind * k, int x, int y, int x2, int y2);
Object* render_add_particle(Kind * k, int x, int y);
void object_prepare_creature(Object * self);
Data3DModel* object_model(Object * self);
void draw_object(Object * ob, Land4x4Matrix tm);
void draw_equipment(Object * ob, Data3DModel * k, int t, float light);
void draw_model(Data3DModel * model, int t, float light);
void draw_shadow(Object * ob, Land4x4Matrix tm);
void draw_object_ui(Object * ob);
void draw_healthbar(Object * ob);
void draw_manabar(Object * ob);
void render_redo_static_objects(void);
void render_create_objects(void);
void get_hex_under_mouse(float mx, float my, int * hx, int * hy);
void get_hex_pixelpos(int x, int y, float * px, float * py);
void render_debug_grid(void);
void box(str name, str format, ...);
void render_infobar(void);
Shadows* shadows_new(Data3DModel * model, Object * equip_ob);
void shadows_maybe_destroy(Shadows * self);
#endif
